@@ -107,7 +107,6 @@ class UpnpFactory:
         self, device_desc_el: ET.Element, description_url: str
     ) -> DeviceInfo:
         """Parse device description XML."""
-        # pylint: disable=no-self-use
         icons = []
         for icon_el in device_desc_el.iterfind("./device:iconList/device:icon", NS):
             icon_url = icon_el.findtext("./device:url", "", NS)
@@ -125,13 +124,21 @@ class UpnpFactory:
             device_type=device_desc_el.findtext("./device:deviceType", "", NS),
             friendly_name=device_desc_el.findtext("./device:friendlyName", "", NS),
             manufacturer=device_desc_el.findtext("./device:manufacturer", "", NS),
-            model_name=device_desc_el.findtext("./device:modelName", "", NS),
-            udn=device_desc_el.findtext("./device:UDN", "", NS),
+            manufacturer_url=device_desc_el.findtext(
+                "./device:manufacturerURL", "", NS
+            ),
             model_description=device_desc_el.findtext(
                 "./device:modelDescription", None, NS
             ),
+            model_name=device_desc_el.findtext("./device:modelName", "", NS),
             model_number=device_desc_el.findtext("./device:modelNumber", None, NS),
+            model_url=device_desc_el.findtext("./device:modelURL", None, NS),
             serial_number=device_desc_el.findtext("./device:serialNumber", None, NS),
+            udn=device_desc_el.findtext("./device:UDN", "", NS),
+            upc=device_desc_el.findtext("./device:UPC", "", NS),
+            presentation_url=device_desc_el.findtext(
+                "./device:presentationURL", "", NS
+            ),
             url=description_url,
             icons=icons,
             xml=device_desc_el,
@@ -163,7 +170,6 @@ class UpnpFactory:
 
     def _parse_service_el(self, service_description_el: ET.Element) -> ServiceInfo:
         """Parse service description XML."""
-        # pylint: disable=no-self-use
         return ServiceInfo(
             service_id=service_description_el.findtext("device:serviceId", "", NS),
             service_type=service_description_el.findtext("device:serviceType", "", NS),
@@ -203,8 +209,6 @@ class UpnpFactory:
         self, state_variable_el: ET.Element
     ) -> StateVariableInfo:
         """Parse XML for state variable."""
-        # pylint: disable=no-self-use
-
         # send events
         send_events = False
         if "sendEvents" in state_variable_el.attrib:
@@ -342,8 +346,6 @@ class UpnpFactory:
 
     def _parse_action_el(self, action_el: ET.Element) -> ActionInfo:
         """Parse XML for action."""
-        # pylint: disable=no-self-use
-
         # build arguments
         args: List[ActionArgumentInfo] = []
         for argument_el in action_el.findall(
