@@ -47,9 +47,11 @@ STATE_VARIABLE_TYPE_MAPPING: Mapping[str, Mapping[str, Callable]] = {
     "ui1": {"type": int, "in": int, "out": str},
     "ui2": {"type": int, "in": int, "out": str},
     "ui4": {"type": int, "in": int, "out": str},
+    "ui8": {"type": int, "in": int, "out": str},
     "i1": {"type": int, "in": int, "out": str},
     "i2": {"type": int, "in": int, "out": str},
     "i4": {"type": int, "in": int, "out": str},
+    "i8": {"type": int, "in": int, "out": str},
     "int": {"type": int, "in": int, "out": str},
     "r4": {"type": float, "in": float, "out": str},
     "r8": {"type": float, "in": float, "out": str},
